import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter_compute_tree/flutter_compute_tree.dart';

import 'src/app.dart';

final _r = Random();

class InvokeAsyncApp extends App {
  const InvokeAsyncApp({super.key});

  @override
  String get name => 'n.invoke.async()';

  @override
  Widget build(CTNode n) {
    final dep = n.ref(() => 0);

    final result = n.invoke.async(
      () => Future.delayed(
        const Duration(seconds: 1),
        () => _r.nextInt(255),
      ),
      dep: dep.value,
    );

    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        if (result is Loading)
          const CircularProgressIndicator()
        else if (result is Failure)
          Text(
            'Error: ${result.failure().exception}',
            style: Theme.of(n.context).textTheme.bodyMedium,
          )
        else if (result is Success)
          Text(
            'Result: ${result.success().value}',
            style: Theme.of(n.context).textTheme.bodyMedium,
          ),
        const SizedBox(height: 16),
        ElevatedButton(
          onPressed: () {
            dep.value++;
          },
          child: const Text('Refresh'),
        ),
      ],
    );
  }
}
