part of 'lib.dart';

abstract class Token<T> {
  const Token();

  Type get contentType => T;
}

const dynamic _constDep = Object();

class TypeToken<T> extends Token<T> {
  static final Map<Type, TypeToken> _typeTokens = {};

  const TypeToken._();

  factory TypeToken() {
    if (_typeTokens.containsKey(T)) {
      return _typeTokens[T] as TypeToken<T>;
    }

    final t = TypeToken<T>._();
    _typeTokens[T] = t;

    return t;
  }
}

abstract class CTNode {
  BuildContext get context;
  late final _units = Units(this);

  T _exec<T>(T Function() fn) {
    _units.enter();
    final v = fn();
    _units.exit();
    return v;
  }

  void _signalUpdate();

  void _provide<T>(Token<T> token, Object entity);
  T _consume<T, K>([Token<T>? token]);
  void _removeProviderFor(Object entity);

  Ref<T> ref<T, K extends Object>(T Function() factory, [K? dep]) {
    dep ??= _constDep;
    return _units.next(() => _RefUnit<T, K>(factory), dep as K);
  }

  T memo<T>(T value) {
    return _units.next<T, Object>(() => _MemoUnit(value), const Object());
  }

  Invoke get invoke => Invoke._(_units);

  Trigger<T> trigger<T extends Object>() {
    return _units.next<Trigger<T>, Object>(
      () => _TriggerUnit<T>(this),
      const Object(),
    );
  }

  T subscribeToRef<T>(Ref<T> ref) {
    if (ref._unit.owner != this) {
      return _units.next(() => _RefSubscriptionUnit(ref), ref);
    } else {
      if (kDebugMode) {
        print('Warning: no need to subscribe to the own ref');
      }

      return ref.value;
    }
  }

  Result<T> subscribeToStream<T, K>(
    Stream<T> Function() streamFactory, {
    T? initialValue,
    K? dep,
  }) {
    return _units.next(
      () => _StreamSubscriptionUnit(
        factory: streamFactory,
        initialResult:
            initialValue == null ? Pending<T>() : Success(initialValue),
      ),
      dep,
    );
  }

  T subscribeToValueListenable<T, K>(
    ValueListenable<T> Function() factory,
    K dep,
  ) {
    return _units.next(() => _ValueListenableUnit<T, K>(factory), dep);
  }

  Ref<T> valueRef<T>(T value) {
    return _units.next(() => _RefUnit(() => value), value);
  }
}
