part of 'lib.dart';

abstract class CTWidget extends Widget {
  const CTWidget({super.key});

  bool get static => false;

  Widget build(CTNode n);

  bool shouldUpdate(
    CTNode node,
    covariant CTWidget oldWidget,
    covariant CTWidget newWidget,
  ) {
    return !static;
  }

  @override
  CTElement createElement() {
    return CTElement(this);
  }
}

class CTElement extends ComponentElement with CTNode {
  CTElement(super.widget);

  @override
  BuildContext get context => this;

  final Set<Object> _providedEntities = {};
  Map<Token, Object>? _entitiesToProvide;
  Map<Object, Token>? _entityToToken;

  @override
  CTWidget get widget => super.widget as CTWidget;

  Widget _build() {
    final w = widget.build(this);
    return w;
  }

  @override
  Widget build() {
    final child = _exec(_build);

    if (_providedEntities.isNotEmpty) {
      return _Provider(
        parent: this,
        entities: _entitiesToProvide!,
        child: child,
      );
    }

    return child;
  }

  @override
  void _provide<T>(Token<T> token, Object entity) {
    _entitiesToProvide ??= {};
    _entityToToken ??= {};
    _providedEntities.add(entity);

    _entitiesToProvide![token] = entity;
    _entityToToken![entity] = token;
  }

  @override
  void _removeProviderFor(Object entity) {
    if (!_providedEntities.contains(entity)) {
      return;
    }

    if (_entityToToken != null) {
      final token = _entityToToken![entity];

      if (token != null) {
        _entitiesToProvide!.remove(token);
        _entityToToken!.remove(ref);
      }
    }
  }

  @override
  T _consume<T, K>([Token<T>? token]) {
    final el = getElementForInheritedWidgetOfExactType<_Provider>()
        as _ProviderElement?;
    final t = token ?? TypeToken<T>();
    final maybeEntity = el?.getEntity(t) ?? Nothing<T>();

    if (maybeEntity is Just<T>) {
      return maybeEntity.value;
    }

    throw Exception('No provider found for ${t.contentType} using $t');
  }

  @override
  void update(CTWidget newWidget) {
    if (widget.shouldUpdate(this, widget, newWidget)) {
      markNeedsBuild();
    }

    super.update(newWidget);
  }

  @override
  void unmount() {
    _units.dispose();
    super.unmount();
  }

  @override
  void _signalUpdate() {
    if (mounted) {
      markNeedsBuild();
    }
  }
}

class _Provider extends InheritedWidget {
  final Map<Token, Object> entities;
  final CTElement parent;

  const _Provider({
    required this.parent,
    required super.child,
    this.entities = const {},
  });

  @override
  InheritedElement createElement() {
    return _ProviderElement(this);
  }

  @override
  bool updateShouldNotify(covariant InheritedWidget oldWidget) {
    return false;
  }
}

class _ProviderElement extends InheritedElement {
  _ProviderElement(super.widget);

  @override
  _Provider get widget => super.widget as _Provider;

  Maybe<T> getEntity<T, K>(Token<K> token) {
    final entity = widget.entities[token];

    if (entity is T) {
      return Just(entity);
    }

    final p = widget.parent;

    final found = p.getElementForInheritedWidgetOfExactType<_Provider>();
    final el = found as _ProviderElement?;

    return el?.getEntity<T, K>(token) ?? Nothing<T>();
  }
}
