part of '../lib.dart';

class _StreamSubscriptionUnit<T, K> extends Unit<Result<T>, K> {
  Result<T> _result;
  late Stream<T> _stream;
  late StreamSubscription<T> _subscription;

  final Stream<T> Function() factory;

  _StreamSubscriptionUnit({
    Result<T>? initialResult,
    required this.factory,
  }) : _result = initialResult ?? Pending<T>();

  @override
  void init(CTNode node, K dep) {
    super.init(node, dep);

    _stream = factory();
    _subscription = _stream.listen(_onValue, onError: _onError);
  }

  void _onValue(T value) {
    _result = Success(value);
    notify();
  }

  void _onError(Object? error) {
    _result = Failure(error);
    notify();
  }

  @override
  void dispose() {
    _subscription.cancel();
    super.dispose();
  }

  @override
  Result<T> unwrap() {
    return _result;
  }
}
