part of '../lib.dart';

abstract class RefToken<T> extends Token<T> {
  const RefToken();
}

class Ref<T> {
  final _RefUnit _unit;
  T _value;

  static Ref<T> consume<T>(CTNode n, [Token<Ref<T>>? token]) {
    return n._consume<Ref<T>, T>(token);
  }

  Ref._(this._unit, this._value);

  T get value => _value;
  set value(T value) => write(value);

  final Set<CTNode> _dependencies = {};

  void write(T value, {bool forceUpdate = false}) {
    if (forceUpdate || _value != value) {
      _value = value;
      _unit.notify();

      for (var n in _dependencies) {
        n._signalUpdate();
      }
    }
  }

  T update(T Function(T) fn, {bool force = false}) {
    final v = fn(_value);

    if (force || _value != v) {
      write(v, forceUpdate: force);
    }

    return v;
  }

  Trigger<K> action<K extends Object>(T Function(T value, K arg) fn) {
    final t = _unit.owner.trigger<K>();
    t.handler((arg) => update((v) => fn(v, arg)), value);

    return t;
  }

  void provide([Token<Ref<T>>? token]) {
    _unit.owner._provide(token ?? TypeToken<Ref<T>>(), this);
  }
}

class _RefUnit<T, K> extends Unit<Ref<T>, K> {
  late final Ref<T> ref;
  final T Function() _factory;

  _RefUnit(this._factory);

  @override
  void init(CTNode node, K dep) {
    super.init(node, dep);
    ref = Ref._(this, _factory());
  }

  @override
  Unit<Ref<T>, K> update(
    CTNode node,
    K oldDep,
    K newDep,
    _RefUnit<T, K> newUnit,
  ) {
    super.update(node, oldDep, newDep, newUnit);

    if (oldDep == newDep) {
      return this;
    }

    ref.value = newUnit._factory();
    return this;
  }

  @override
  Ref<T> unwrap() => ref;

  @override
  void dispose() {
    owner._removeProviderFor(ref);
    super.dispose();
  }
}
